package net.universe_factory.minecraft.test;

import java.util.AbstractMap;
import java.util.ArrayList;
import java.util.Map.Entry;
import java.util.Random;

import net.minecraft.block.Block;
import net.minecraft.init.Blocks;
import net.minecraft.init.Items;
import net.minecraft.item.Item;
import net.minecraft.item.ItemCloth;
import net.minecraft.item.ItemFood;
import net.minecraft.item.ItemSeedFood;
import net.minecraft.item.ItemSeeds;
import net.minecraft.item.ItemStack;
import net.minecraft.world.World;
import net.minecraft.world.biome.BiomeGenBase;
import net.minecraft.world.gen.feature.WorldGenAbstractTree;
import net.minecraft.world.gen.feature.WorldGenBigTree;
import net.minecraft.world.gen.feature.WorldGenTrees;
import net.minecraftforge.common.MinecraftForge;
import net.minecraftforge.event.terraingen.BiomeEvent.CreateDecorator;
import net.universe_factory.minecraft.test.blocks.BlockGlowfoo;
import net.universe_factory.minecraft.test.generic.GenericCrops;
import net.universe_factory.minecraft.test.generic.GenericWood;
import net.universe_factory.minecraft.test.generic.WorldGenGenericBigTree;
import net.universe_factory.minecraft.test.generic.WorldGenGenericTrees;
import net.universe_factory.minecraft.test.world.ExtensibleBiomeDecorator;
import cpw.mods.fml.common.Mod;
import cpw.mods.fml.common.Mod.EventHandler;
import cpw.mods.fml.common.Mod.Instance;
import cpw.mods.fml.common.event.FMLInitializationEvent;
import cpw.mods.fml.common.eventhandler.SubscribeEvent;
import cpw.mods.fml.common.registry.GameRegistry;

@Mod(modid = Test.MODID, version = Test.VERSION)
public class Test {
	public static final String MODID = "neoraider_test";
	public static final String VERSION = "1";

	@Instance(value = MODID)
	public static Test instance;

	public static final Item cherries = new ItemFood(2, 0.2f, false).setTextureName(MODID + ":cherries").setUnlocalizedName("cherries");
	public static final GenericWood cherryTree = new GenericWood() {
		@Override
		public String getName() {
			return "Cherry";
		}

		@Override
		public int getMinTreeHeight() {
			return 4;
		}

		@Override
		public ArrayList<ItemStack> getDrops(World world, int x, int y, int z, int meta, int fortune) {
			ArrayList<ItemStack> ret = new ArrayList<ItemStack>();

			int chance = 30;

			if (fortune > 0) {
				chance -= 2 << fortune;

				if (chance < 5)
					chance = 5;
			}

			if (world.rand.nextInt(chance) == 0)
				ret.add(new ItemStack(cherries, 1));

			return ret;
		}
	};

	public static final GenericWood schnitzelTree = new GenericWood() {
		@Override
		public String getName() {
			return "Schnitzel";
		}

		@Override
		public int getMinTreeHeight() {
			return 4;
		}

		@Override
		public ArrayList<ItemStack> getDrops(World world, int x, int y, int z, int meta, int fortune) {
			ArrayList<ItemStack> ret = new ArrayList<ItemStack>();

			int chance = 30;

			if (fortune > 0) {
				chance -= 2 << fortune;

				if (chance < 5)
					chance = 5;
			}

			if (world.rand.nextInt(chance) == 0) {
				final Item[] drops = { Items.beef, Items.beef, Items.chicken, Items.chicken, Items.porkchop, Items.porkchop, Items.leather };
				ret.add(new ItemStack(drops[world.rand.nextInt(drops.length)], 1));
			}

			return ret;
		}
	};

	WorldGenGenericTrees cherryTreeGenerator = new WorldGenGenericTrees(false, cherryTree);
	WorldGenGenericBigTree cherryTreeGeneratorBig = new WorldGenGenericBigTree(false, cherryTree);

	public static final GenericCrops saladField = new GenericCrops() {
		@Override
		public String getName() {
			return "salad";
		}

		@Override
		public Item itemCrops() {
			return salad;
		}

		@Override
		public Item itemSeeds() {
			return saladSeeds;
		}

		@Override
		public int getRenderType() {
			return 1;
		}
	};
	public static final Item saladSeeds = new ItemSeeds(saladField, Blocks.farmland).setTextureName(MODID + ":seeds_salad")
			.setUnlocalizedName("seedsSalad");
	public static final Item salad = new ItemFood(3, 0.8f, false).setTextureName(MODID + ":salad").setUnlocalizedName("salad");

	public static final GenericCrops cucumberField = new GenericCrops() {
		@Override
		public String getName() {
			return "cucumber";
		}

		@Override
		public Item itemCrops() {
			return cucumber;
		}

		@Override
		public Item itemSeeds() {
			return cucumberSeeds;
		}

		@Override
		public int getRenderType() {
			return 1;
		}
	};
	public static final Item cucumberSeeds = new ItemSeeds(cucumberField, Blocks.farmland).setTextureName(MODID + ":seeds_cucumber").setUnlocalizedName(
			"seedsCucumber");
	public static final Item cucumber = new ItemFood(4, 0.4f, false).setTextureName(MODID + ":cucumber").setUnlocalizedName("cucumber");

	public static final GenericCrops raddishField = new GenericCrops() {
		@Override
		public String getName() {
			return "raddish";
		}

		@Override
		public Item itemCrops() {
			return raddish;
		}

		@Override
		public Item itemSeeds() {
			return raddish;
		}
	};
	public static final Item raddish = new ItemSeedFood(4, 0.4f, raddishField, Blocks.farmland).setTextureName(MODID + ":raddish").setUnlocalizedName(
			"raddish");

	public static final Block glowfooBlock = new BlockGlowfoo();
	public static final Item glowfoo = new ItemCloth(glowfooBlock);

	public static final ExtensibleBiomeDecorator biomeDecorator = new ExtensibleBiomeDecorator();

	public static final LiquidLight liquidLight = new LiquidLight();

	@EventHandler
	public void preInit(FMLInitializationEvent event) {
		MinecraftForge.TERRAIN_GEN_BUS.register(new TerrainGenHandler());

		GameRegistry.registerItem(cherries, "cherries");
		cherryTree.register();

		schnitzelTree.register();

		GameRegistry.registerItem(saladSeeds, "seeds_salad");
		GameRegistry.registerItem(salad, "salad");
		GameRegistry.registerBlock(saladField, null, "salad");

		GameRegistry.registerItem(cucumberSeeds, "seeds_cucumber");
		GameRegistry.registerItem(cucumber, "cucumber");
		GameRegistry.registerBlock(cucumberField, null, "cucumber");

		GameRegistry.registerItem(raddish, "raddish");
		GameRegistry.registerBlock(raddishField, null, "raddish");

		GameRegistry.registerBlock(glowfooBlock, null, "glowfoo");
		GameRegistry.registerItem(glowfoo, "glowfoo");

		for (int i = 0; i < 16; i++)
			GameRegistry.addRecipe(new ItemStack(glowfoo, 4, i), "###", "#X#", "###", '#', new ItemStack(Blocks.stained_glass_pane, 1, i), 'X',
					Blocks.redstone_block);

		liquidLight.register();

		biomeDecorator.registerTree(new ExtensibleBiomeDecorator.Tree() {
			@Override
			public Entry<WorldGenAbstractTree, Float> replaceTree(World world, BiomeGenBase biome, Random random, int x, int y, int z,
					WorldGenAbstractTree orig) {
				if (!(orig instanceof WorldGenTrees || orig instanceof WorldGenBigTree)
						|| biome.getTempCategory() != BiomeGenBase.TempCategory.MEDIUM)
					return null;

				WorldGenAbstractTree generator = (random.nextInt(3) == 0) ? cherryTreeGeneratorBig : cherryTreeGenerator;

				return new AbstractMap.SimpleEntry(generator, 0.3f);
			}
		});
	}

	public class TerrainGenHandler {
		@SubscribeEvent
		public void handleCreateDecorator(CreateDecorator event) {
			event.newBiomeDecorator = biomeDecorator.getInstance(event.originalBiomeDecorator);
		}
	}
}
