package net.universe_factory.minecraft.test.generic;

import java.util.Random;

import net.minecraft.block.Block;
import net.minecraft.world.World;
import net.minecraft.world.gen.feature.WorldGenAbstractTree;
import net.minecraftforge.common.util.ForgeDirection;

public class WorldGenGenericTrees extends WorldGenAbstractTree {
	private final GenericWood genericWood;

	public WorldGenGenericTrees(boolean notify, GenericWood genericWood) {
		super(notify);

		this.genericWood = genericWood;
	}

	@Override
	public boolean generate(World world, Random random, int x, int y, int z) {
		int height = random.nextInt(3) + genericWood.getMinTreeHeight();
		boolean flag = true;

		if (y >= 1 && y + height + 1 <= 256) {
			byte b0;
			int k1;
			Block block;

			for (int i1 = y; i1 <= y + 1 + height; ++i1) {
				b0 = 1;

				if (i1 == y) {
					b0 = 0;
				}

				if (i1 >= y + 1 + height - 2) {
					b0 = 2;
				}

				for (int j1 = x - b0; j1 <= x + b0 && flag; ++j1) {
					for (k1 = z - b0; k1 <= z + b0 && flag; ++k1) {
						if (i1 >= 0 && i1 < 256) {
							block = world.getBlock(j1, i1, k1);

							if (!this.isReplaceable(world, j1, i1, k1)) {
								flag = false;
							}
						} else {
							flag = false;
						}
					}
				}
			}

			if (!flag) {
				return false;
			} else {
				Block block2 = world.getBlock(x, y - 1, z);

				boolean isSoil = block2.canSustainPlant(world, x, y - 1, z, ForgeDirection.UP, genericWood.sapling);
				if (isSoil && y < 256 - height - 1) {
					block2.onPlantGrow(world, x, y - 1, z, x, y, z);
					b0 = 3;
					byte b1 = 0;
					int l1;
					int i2;
					int j2;
					int i3;

					for (k1 = y - b0 + height; k1 <= y + height; ++k1) {
						i3 = k1 - (y + height);
						l1 = b1 + 1 - i3 / 2;

						for (i2 = x - l1; i2 <= x + l1; ++i2) {
							j2 = i2 - x;

							for (int k2 = z - l1; k2 <= z + l1; ++k2) {
								int l2 = k2 - z;

								if (Math.abs(j2) != l1 || Math.abs(l2) != l1 || random.nextInt(2) != 0 && i3 != 0) {
									Block block1 = world.getBlock(i2, k1, k2);

									if (block1.isAir(world, i2, k1, k2) || block1.isLeaves(world, i2, k1, k2)) {
										this.setBlockAndNotifyAdequately(world, i2, k1, k2, genericWood.leaves, 0);
									}
								}
							}
						}
					}

					for (k1 = 0; k1 < height; ++k1) {
						block = world.getBlock(x, y + k1, z);

						if (block.isAir(world, x, y + k1, z) || block.isLeaves(world, x, y + k1, z)) {
							this.setBlockAndNotifyAdequately(world, x, y + k1, z, genericWood.log, 0);

						}
					}

					return true;
				} else {
					return false;
				}
			}
		} else {
			return false;
		}
	}

}
