package net.universe_factory.minecraft.test.world;

import static net.minecraftforge.event.terraingen.DecorateBiomeEvent.Decorate.EventType.BIG_SHROOM;
import static net.minecraftforge.event.terraingen.DecorateBiomeEvent.Decorate.EventType.CACTUS;
import static net.minecraftforge.event.terraingen.DecorateBiomeEvent.Decorate.EventType.CLAY;
import static net.minecraftforge.event.terraingen.DecorateBiomeEvent.Decorate.EventType.DEAD_BUSH;
import static net.minecraftforge.event.terraingen.DecorateBiomeEvent.Decorate.EventType.FLOWERS;
import static net.minecraftforge.event.terraingen.DecorateBiomeEvent.Decorate.EventType.GRASS;
import static net.minecraftforge.event.terraingen.DecorateBiomeEvent.Decorate.EventType.LAKE;
import static net.minecraftforge.event.terraingen.DecorateBiomeEvent.Decorate.EventType.LILYPAD;
import static net.minecraftforge.event.terraingen.DecorateBiomeEvent.Decorate.EventType.PUMPKIN;
import static net.minecraftforge.event.terraingen.DecorateBiomeEvent.Decorate.EventType.REED;
import static net.minecraftforge.event.terraingen.DecorateBiomeEvent.Decorate.EventType.SAND;
import static net.minecraftforge.event.terraingen.DecorateBiomeEvent.Decorate.EventType.SAND_PASS2;
import static net.minecraftforge.event.terraingen.DecorateBiomeEvent.Decorate.EventType.SHROOM;
import static net.minecraftforge.event.terraingen.DecorateBiomeEvent.Decorate.EventType.TREE;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;
import java.util.Random;

import net.minecraft.block.BlockFlower;
import net.minecraft.block.material.Material;
import net.minecraft.init.Blocks;
import net.minecraft.world.World;
import net.minecraft.world.biome.BiomeDecorator;
import net.minecraft.world.biome.BiomeGenBase;
import net.minecraft.world.gen.feature.WorldGenAbstractTree;
import net.minecraft.world.gen.feature.WorldGenDeadBush;
import net.minecraft.world.gen.feature.WorldGenLiquids;
import net.minecraft.world.gen.feature.WorldGenPumpkin;
import net.minecraft.world.gen.feature.WorldGenerator;
import net.minecraftforge.common.MinecraftForge;
import net.minecraftforge.event.terraingen.DecorateBiomeEvent;
import net.minecraftforge.event.terraingen.TerrainGen;

public class ExtensibleBiomeDecorator {
	private ArrayList<Tree> trees = new ArrayList<Tree>();

	public void registerTree(Tree tree) {
		trees.add(tree);
	}

	public BiomeDecorator getInstance(BiomeDecorator base) {
		BiomeDecorator decorator = new Decorator();

		if (base != null) {
			decorator.bigMushroomsPerChunk = base.bigMushroomsPerChunk;
			decorator.cactiPerChunk = base.cactiPerChunk;
			decorator.clayPerChunk = base.clayPerChunk;
			decorator.deadBushPerChunk = base.deadBushPerChunk;
			decorator.flowersPerChunk = base.flowersPerChunk;
			decorator.generateLakes = base.generateLakes;
			decorator.grassPerChunk = base.grassPerChunk;
			decorator.mushroomsPerChunk = base.mushroomsPerChunk;
			decorator.reedsPerChunk = base.reedsPerChunk;
			decorator.sandPerChunk = base.sandPerChunk;
			decorator.sandPerChunk2 = base.sandPerChunk2;
			decorator.treesPerChunk = base.treesPerChunk;
			decorator.waterlilyPerChunk = base.waterlilyPerChunk;
		}

		return decorator;
	}

	public interface Tree {
		public Map.Entry<WorldGenAbstractTree, Float> replaceTree(World world, BiomeGenBase biome, Random random, int x, int y, int z,
				WorldGenAbstractTree orig);
	}

	protected class Decorator extends BiomeDecorator {
		protected void genDecorations(BiomeGenBase biome) {
			MinecraftForge.EVENT_BUS.post(new DecorateBiomeEvent.Pre(currentWorld, randomGenerator, chunk_X, chunk_Z));
			this.generateOres();
			int i;
			int j;
			int k;

			boolean doGen = TerrainGen.decorate(currentWorld, randomGenerator, chunk_X, chunk_Z, SAND);
			for (i = 0; doGen && i < this.sandPerChunk2; ++i) {
				j = this.chunk_X + this.randomGenerator.nextInt(16) + 8;
				k = this.chunk_Z + this.randomGenerator.nextInt(16) + 8;
				this.sandGen.generate(this.currentWorld, this.randomGenerator, j, this.currentWorld.getTopSolidOrLiquidBlock(j, k), k);
			}

			doGen = TerrainGen.decorate(currentWorld, randomGenerator, chunk_X, chunk_Z, CLAY);
			for (i = 0; doGen && i < this.clayPerChunk; ++i) {
				j = this.chunk_X + this.randomGenerator.nextInt(16) + 8;
				k = this.chunk_Z + this.randomGenerator.nextInt(16) + 8;
				this.clayGen.generate(this.currentWorld, this.randomGenerator, j, this.currentWorld.getTopSolidOrLiquidBlock(j, k), k);
			}

			doGen = TerrainGen.decorate(currentWorld, randomGenerator, chunk_X, chunk_Z, SAND_PASS2);
			for (i = 0; doGen && i < this.sandPerChunk; ++i) {
				j = this.chunk_X + this.randomGenerator.nextInt(16) + 8;
				k = this.chunk_Z + this.randomGenerator.nextInt(16) + 8;
				this.gravelAsSandGen.generate(this.currentWorld, this.randomGenerator, j, this.currentWorld.getTopSolidOrLiquidBlock(j, k), k);
			}

			i = this.treesPerChunk;

			if (this.randomGenerator.nextInt(10) == 0) {
				++i;
			}

			int l;
			int i1;

			doGen = TerrainGen.decorate(currentWorld, randomGenerator, chunk_X, chunk_Z, TREE);
			for (j = 0; doGen && j < i; ++j) {
				k = this.chunk_X + this.randomGenerator.nextInt(16) + 8;
				l = this.chunk_Z + this.randomGenerator.nextInt(16) + 8;
				i1 = this.currentWorld.getHeightValue(k, l);

				WorldGenAbstractTree defaultTreeGen = biome.func_150567_a(this.randomGenerator);

				float weight = 1.0f;
				Map<WorldGenAbstractTree, Float> treeWeights = new HashMap<WorldGenAbstractTree, Float>();
				treeWeights.put(defaultTreeGen, 1.0f);

				for (Tree tree : trees) {
					Map.Entry<WorldGenAbstractTree, Float> entry = tree.replaceTree(currentWorld, biome, randomGenerator, k, i1, l,
							defaultTreeGen);

					if (entry != null) {
						weight += entry.getValue();
						treeWeights.put(entry.getKey(), entry.getValue());
					}
				}

				WorldGenAbstractTree treeGen = null;
				float p = randomGenerator.nextFloat() * weight;

				for (Map.Entry<WorldGenAbstractTree, Float> entry : treeWeights.entrySet()) {
					p -= entry.getValue();

					if (p < 0) {
						treeGen = entry.getKey();
						break;
					}
				}

				treeGen.setScale(1.0D, 1.0D, 1.0D);

				if (treeGen.generate(this.currentWorld, this.randomGenerator, k, i1, l))
					treeGen.func_150524_b(this.currentWorld, this.randomGenerator, k, i1, l);
			}

			doGen = TerrainGen.decorate(currentWorld, randomGenerator, chunk_X, chunk_Z, BIG_SHROOM);
			for (j = 0; doGen && j < this.bigMushroomsPerChunk; ++j) {
				k = this.chunk_X + this.randomGenerator.nextInt(16) + 8;
				l = this.chunk_Z + this.randomGenerator.nextInt(16) + 8;
				this.bigMushroomGen.generate(this.currentWorld, this.randomGenerator, k, this.currentWorld.getHeightValue(k, l), l);
			}

			doGen = TerrainGen.decorate(currentWorld, randomGenerator, chunk_X, chunk_Z, FLOWERS);
			for (j = 0; doGen && j < this.flowersPerChunk; ++j) {
				k = this.chunk_X + this.randomGenerator.nextInt(16) + 8;
				l = this.chunk_Z + this.randomGenerator.nextInt(16) + 8;
				i1 = this.randomGenerator.nextInt(this.currentWorld.getHeightValue(k, l) + 32);
				String s = biome.func_150572_a(this.randomGenerator, k, i1, l);
				BlockFlower blockflower = BlockFlower.func_149857_e(s);

				if (blockflower.getMaterial() != Material.air) {
					this.yellowFlowerGen.func_150550_a(blockflower, BlockFlower.func_149856_f(s));
					this.yellowFlowerGen.generate(this.currentWorld, this.randomGenerator, k, i1, l);
				}
			}

			doGen = TerrainGen.decorate(currentWorld, randomGenerator, chunk_X, chunk_Z, GRASS);
			for (j = 0; doGen && j < this.grassPerChunk; ++j) {
				k = this.chunk_X + this.randomGenerator.nextInt(16) + 8;
				l = this.chunk_Z + this.randomGenerator.nextInt(16) + 8;
				i1 = this.randomGenerator.nextInt(this.currentWorld.getHeightValue(k, l) * 2);
				WorldGenerator worldgenerator = biome.getRandomWorldGenForGrass(this.randomGenerator);
				worldgenerator.generate(this.currentWorld, this.randomGenerator, k, i1, l);
			}

			doGen = TerrainGen.decorate(currentWorld, randomGenerator, chunk_X, chunk_Z, DEAD_BUSH);
			for (j = 0; doGen && j < this.deadBushPerChunk; ++j) {
				k = this.chunk_X + this.randomGenerator.nextInt(16) + 8;
				l = this.chunk_Z + this.randomGenerator.nextInt(16) + 8;
				i1 = this.randomGenerator.nextInt(this.currentWorld.getHeightValue(k, l) * 2);
				(new WorldGenDeadBush(Blocks.deadbush)).generate(this.currentWorld, this.randomGenerator, k, i1, l);
			}

			doGen = TerrainGen.decorate(currentWorld, randomGenerator, chunk_X, chunk_Z, LILYPAD);
			for (j = 0; doGen && j < this.waterlilyPerChunk; ++j) {
				k = this.chunk_X + this.randomGenerator.nextInt(16) + 8;
				l = this.chunk_Z + this.randomGenerator.nextInt(16) + 8;

				for (i1 = this.randomGenerator.nextInt(this.currentWorld.getHeightValue(k, l) * 2); i1 > 0
						&& this.currentWorld.isAirBlock(k, i1 - 1, l); --i1) {
					;
				}

				this.waterlilyGen.generate(this.currentWorld, this.randomGenerator, k, i1, l);
			}

			doGen = TerrainGen.decorate(currentWorld, randomGenerator, chunk_X, chunk_Z, SHROOM);
			for (j = 0; doGen && j < this.mushroomsPerChunk; ++j) {
				if (this.randomGenerator.nextInt(4) == 0) {
					k = this.chunk_X + this.randomGenerator.nextInt(16) + 8;
					l = this.chunk_Z + this.randomGenerator.nextInt(16) + 8;
					i1 = this.currentWorld.getHeightValue(k, l);
					this.mushroomBrownGen.generate(this.currentWorld, this.randomGenerator, k, i1, l);
				}

				if (this.randomGenerator.nextInt(8) == 0) {
					k = this.chunk_X + this.randomGenerator.nextInt(16) + 8;
					l = this.chunk_Z + this.randomGenerator.nextInt(16) + 8;
					i1 = this.randomGenerator.nextInt(this.currentWorld.getHeightValue(k, l) * 2);
					this.mushroomRedGen.generate(this.currentWorld, this.randomGenerator, k, i1, l);
				}
			}

			if (doGen && this.randomGenerator.nextInt(4) == 0) {
				j = this.chunk_X + this.randomGenerator.nextInt(16) + 8;
				k = this.chunk_Z + this.randomGenerator.nextInt(16) + 8;
				l = this.randomGenerator.nextInt(this.currentWorld.getHeightValue(j, k) * 2);
				this.mushroomBrownGen.generate(this.currentWorld, this.randomGenerator, j, l, k);
			}

			if (doGen && this.randomGenerator.nextInt(8) == 0) {
				j = this.chunk_X + this.randomGenerator.nextInt(16) + 8;
				k = this.chunk_Z + this.randomGenerator.nextInt(16) + 8;
				l = this.randomGenerator.nextInt(this.currentWorld.getHeightValue(j, k) * 2);
				this.mushroomRedGen.generate(this.currentWorld, this.randomGenerator, j, l, k);
			}

			doGen = TerrainGen.decorate(currentWorld, randomGenerator, chunk_X, chunk_Z, REED);
			for (j = 0; doGen && j < this.reedsPerChunk; ++j) {
				k = this.chunk_X + this.randomGenerator.nextInt(16) + 8;
				l = this.chunk_Z + this.randomGenerator.nextInt(16) + 8;
				i1 = this.randomGenerator.nextInt(this.currentWorld.getHeightValue(k, l) * 2);
				this.reedGen.generate(this.currentWorld, this.randomGenerator, k, i1, l);
			}

			for (j = 0; doGen && j < 10; ++j) {
				k = this.chunk_X + this.randomGenerator.nextInt(16) + 8;
				l = this.chunk_Z + this.randomGenerator.nextInt(16) + 8;
				i1 = this.randomGenerator.nextInt(this.currentWorld.getHeightValue(k, l) * 2);
				this.reedGen.generate(this.currentWorld, this.randomGenerator, k, i1, l);
			}

			doGen = TerrainGen.decorate(currentWorld, randomGenerator, chunk_X, chunk_Z, PUMPKIN);
			if (doGen && this.randomGenerator.nextInt(32) == 0) {
				j = this.chunk_X + this.randomGenerator.nextInt(16) + 8;
				k = this.chunk_Z + this.randomGenerator.nextInt(16) + 8;
				l = this.randomGenerator.nextInt(this.currentWorld.getHeightValue(j, k) * 2);
				(new WorldGenPumpkin()).generate(this.currentWorld, this.randomGenerator, j, l, k);
			}

			doGen = TerrainGen.decorate(currentWorld, randomGenerator, chunk_X, chunk_Z, CACTUS);
			for (j = 0; doGen && j < this.cactiPerChunk; ++j) {
				k = this.chunk_X + this.randomGenerator.nextInt(16) + 8;
				l = this.chunk_Z + this.randomGenerator.nextInt(16) + 8;
				i1 = this.randomGenerator.nextInt(this.currentWorld.getHeightValue(k, l) * 2);
				this.cactusGen.generate(this.currentWorld, this.randomGenerator, k, i1, l);
			}

			doGen = TerrainGen.decorate(currentWorld, randomGenerator, chunk_X, chunk_Z, LAKE);
			if (doGen && this.generateLakes) {
				for (j = 0; j < 50; ++j) {
					k = this.chunk_X + this.randomGenerator.nextInt(16) + 8;
					l = this.randomGenerator.nextInt(this.randomGenerator.nextInt(248) + 8);
					i1 = this.chunk_Z + this.randomGenerator.nextInt(16) + 8;
					(new WorldGenLiquids(Blocks.flowing_water)).generate(this.currentWorld, this.randomGenerator, k, l, i1);
				}

				for (j = 0; j < 20; ++j) {
					k = this.chunk_X + this.randomGenerator.nextInt(16) + 8;
					l = this.randomGenerator.nextInt(this.randomGenerator.nextInt(this.randomGenerator.nextInt(240) + 8) + 8);
					i1 = this.chunk_Z + this.randomGenerator.nextInt(16) + 8;
					(new WorldGenLiquids(Blocks.flowing_lava)).generate(this.currentWorld, this.randomGenerator, k, l, i1);
				}
			}

			MinecraftForge.EVENT_BUS.post(new DecorateBiomeEvent.Post(currentWorld, randomGenerator, chunk_X, chunk_Z));
		}
	}
}
